"""
Module decomposition numbers of theorem of Lagrange
"""


def lagrange(num: int) -> tuple:
    """
    Function of decomposition numbers into sum of four squares
    :param num: Input number
    :return: Tuple of decomposition number
    """
    maximum = int(num ** 0.5)
    for i in range(maximum + 1):
        for j in range(maximum + 1):
            for k in range(maximum + 1):
                for l in range(maximum + 1):
                    if i ** 2 + j ** 2 + k ** 2 + l ** 2 == num:
                        return l, k, j, i


def calculate_numbers(left_border: int, right_border: int) -> list:
    """
    Function decompose number in interval [a, b)
    :param left_border:
    :param right_border:
    :return:
    """
    list_of_numbers = []
    for i in range(left_border, right_border):
        list_of_numbers.append(
            tuple([i, lagrange(i)]))

    return list_of_numbers



"""
This module is the entry point
"""


import argparse
import doctest
import pytest
from input_interval import set_interval
from calculate_numbers import calculate_numbers


def start_program() -> None:
    """
    Function decomposition input numbers and print his
    :return: None
    """
    while (command := input('Enter any symbol to continue '
                            '"stop" ')) != 'stop':
        first_number, second_number = set_interval()
        if first_number is not None and second_number is not None:
            for num, decomp in calculate_numbers(first_number, second_number):
                print(f'{num} -> {decomp}')


def main() -> None:
    """
    Entry point of program
    :return: None
    """
    parser = argparse.ArgumentParser(description='Decomposition numbers')
    parser.add_argument('--mode', type=str, dest='mode', default='input',
                        help='Select mode of working the program')

    args = parser.parse_args()
    if args.mode == 'pytest':
        pytest.main([r"package\tests\tests.py"])
    elif args.mode == 'doctest':
        doctest.testfile(r'tests/test_calculate_numbers.txt', verbose='-v')
    elif args.mode == 'input':
        start_program()


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name='lagrange_theorem',
    version='1.0.1',
    author='Dima Skorobogatov',
    author_email='dima.skorobogatov.99@mail.ru',
    description='Decomposition of numbers into the sum of four squares',
    long_description=open('README.txt').read(),
    packages=find_packages(),
    install_requires=['pytest']
)



"""
Module sets the interval of numbers
"""


def set_interval() -> tuple:
    left_border_default, right_border_default = 1, 11
    left_border, right_border = None, None
    try:
        interval = list(map(int, input().split()))
    except ValueError:
        print('Input wrong value!')
        return left_border, right_border

    if len(interval) == 1:
        left_border = 0
        right_border = interval[0]
    elif len(interval) > 2:
        print('Input many values!')
        return left_border, right_border
    elif interval[0] < 0 or interval[1] < 0:
        print('Input values cannot be negative')
        return left_border, right_border
    elif len(interval) == 0:
        left_border = left_border_default
        right_border = right_border_default
    elif interval[0] >= interval[1]:
        print('The second number cannot be less than the first!')
        return left_border, right_border
    elif len(interval) == 2:
        left_border = interval[0]
        right_border = interval[1]

    return left_border, right_border
